(function(angular, $, _, L) {
  "use strict";

  angular.module('crmSearchDisplayMapLeaflet').component('crmSearchDisplayMapLeaflet', {
    bindings: {
      apiEntity: '@',
      search: '<',
      display: '<',
      apiParams: '<',
      settings: '<',
      filters: '<',
      totalCount: '=?'
    },
    require: {
      afFieldset: '?^^afFieldset'
    },
    templateUrl: '~/crmSearchDisplayMapLeaflet/crmSearchDisplayMapLeaflet.html',
    controller: function($scope, $element, searchDisplayBaseTrait) {
      const ts = $scope.ts = CRM.ts('org.civicrm.search_kit'),

      // Mix in required traits
      ctrl = angular.extend(this, _.cloneDeep(searchDisplayBaseTrait));

      let map = null;

      this.$onInit = function() {
        this.initializeDisplay($scope, $element);

        const displayElement = $element[0];
        const mapElement = displayElement.getElementsByClassName('crm-search-kit-map')[0];
        mapElement.style.height = (ctrl.settings.format.height + 'px');
        mapElement.style.width = (ctrl.settings.format.width + 'px');

        map = L.map(mapElement, {
          renderer: L.canvas()
        });

        L.tileLayer('https://tile.openstreetmap.org/{z}/{x}/{y}.png', {
          maxZoom: 19,
          attribution: '&copy; <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>'
        }).addTo(map);
      };

      let markersLayerGroup = L.layerGroup();

      this.onPostRun.push(function() {
        if (map.hasLayer(markersLayerGroup)) {
          markersLayerGroup.clearLayers();
        }

        if (!ctrl.results) {
          return;
        }

        if (ctrl.settings.cluster) {
          markersLayerGroup = L.markerClusterGroup({showCoverageOnHover: false});
        }

        const markerData = ctrl.results.map(function (record) {
          return {
            'lat': record.data[ctrl.settings.columns[0].key],
            'lon': record.data[ctrl.settings.columns[1].key],
            'popupHtml': renderPopupHtml(record),
          };
        });

        function renderPopupHtml(record) {
          const colData = record.columns[2];
          let html = colData.val;
          if (colData.icons) {
            if (colData.icons.left) {
              html = '<i class="crm-i ' + colData.icons.left[0] + '"></i> ' + html;
            }
            if (colData.icons.right) {
              html += ' <i class="crm-i ' + colData.icons.right[0] + '"></i>';
            }
          }
          return html;
        }

        const latlngs = ctrl.results.map(function (record) {
          return L.latLng(record.data[ctrl.settings.columns[0].key], record.data[ctrl.settings.columns[1].key]);
        });

        L.Icon.Default.imagePath = CRM.crmSearchDisplayMapLeaflet.basePath + '/images/';

        for (var i = 0; i < markerData.length; i++) {
          if ((markerData[i].lat === null) || (markerData[i].lon === null)) {
            continue;
          }
          const marker = new L.marker([markerData[i].lat,markerData[i].lon])
            .bindPopup(markerData[i].popupHtml);
          markersLayerGroup.addLayer(marker);
        }
        map.addLayer(markersLayerGroup);
        try { map.fitBounds(latlngs); } catch (err) { console.log('Invalid Lat/Lon coordinates: check you search data'); }

      });
    }
   });
})(angular, CRM.$, CRM._, CRM.searchkitmaps.leaflet);
